import os
import re
import sys
import inspect
from importlib import import_module


def get_flagged_classes(module_name, function):
    """\
    Iterate over module and return classes that return true to the provided function.
    """
    module = sys.modules[module_name]
    for name, obj in inspect.getmembers(module):
        if inspect.isclass(obj) and function(obj):
            yield obj


class CallbackLoader:
    """\
    Load modules from a file path. You can optionally add callbacks which
    test against a function which are then passed to the callback.
    """

    # File name of a module:
    __module_file_regexp = "(.+)\.py(c?)$"

    def __init__(self):
        self.callbacks = {}

    def add_callback(self, name, function, callback):
        self.callbacks[name] = (function, callback)

    def load(self, path, env):
        """\
        Load modules from the provided path and if any classes contain the attr_flag
        the pass the class to the callback.
        """

        # Add the path to system path
        sys.path.append(path)

        module_names = set()

        for entry in os.listdir(path):
            if os.path.isfile(os.path.join(path, entry)):
                regexp_result = re.search(self.__module_file_regexp, entry)
                if regexp_result:  # is a module file name
                    module_names.add(regexp_result.groups()[0])

        for module_name in sorted(module_names):  # for each found module...
            env[module_name] = import_module(module_name)
            for name in self.callbacks:
                (function, callback) = self.callbacks[name]
                for cls in get_flagged_classes(module_name, function):
                    callback(cls)
